#include "MainWindow.h"
#include <QtDBus>
#include <QServiceManager>

#include <iostream>
using namespace std;

#include <TelepathyQt4/PendingStringList>
#include <TelepathyQt4/PendingReady>

QTM_USE_NAMESPACE
Q_DECLARE_METATYPE(QServiceInterfaceDescriptor)

MainWindow::MainWindow (QWidget *parent/* = 0*/)
: QMainWindow (parent)
, txtLogs (NULL)
, grid (NULL)
{
    clientRegistrar = ClientRegistrar::create();

    QWidget *central = new QWidget (this);

    btnDoText   = new QCheckBox ("Monitor text", central);
    btnDoStream = new QCheckBox ("Monitor streamed media", central);

    btnAccept   = new QRadioButton ("Always Accept", central);
    btnIgnore   = new QRadioButton ("Always Ignore", central);
    btnReject   = new QRadioButton ("Always Reject", central);

    btnActionGroup = new QButtonGroup (central);
    btnActionGroup->addButton (btnIgnore, 1);
    btnActionGroup->addButton (btnAccept, 2);
    btnActionGroup->addButton (btnReject, 3);
    btnIgnore->setChecked (true);   // Default is ignore

    txtLogs     = new QPlainTextEdit (central);
    grid        = new QGridLayout (central);

    QObject::connect (btnDoText, SIGNAL (clicked ()),
                      this     , SLOT   (onDoBtnClicked ()));
    QObject::connect (btnDoStream, SIGNAL (clicked ()),
                      this       , SLOT   (onDoBtnClicked ()));
    QObject::connect (btnActionGroup, SIGNAL (buttonClicked (int)),
                      this          , SLOT   (onActionSelected (int)));


    grid->addWidget (btnDoText  , 0,0, 1,4);
    grid->addWidget (btnDoStream, 1,0, 1,4);
    grid->addWidget (btnIgnore  , 2,0, 1,4);
    grid->addWidget (btnAccept  , 3,0, 1,4);
    grid->addWidget (btnReject  , 4,0, 1,4);
    grid->addWidget (txtLogs    , 5,0, 1,4);

    central->setLayout (grid);
    this->setCentralWidget (central);
}//MainWindow::MainWindow

void
MainWindow::log (const QString &strText, int level/* = 10*/)
{
    QDateTime dt = QDateTime::currentDateTime ();
    QString strLog = QString("%1 : %2 : %3")
                        .arg(dt.toString ("yyyy-MM-dd hh:mm:ss.zzz"))
                        .arg(level)
                        .arg(strText);
    if (NULL != txtLogs)
    {
        txtLogs->appendPlainText (strLog);
    }
    cout << strLog.toStdString () << endl;
}//MainWindow::log

void
MainWindow::setStatus(const QString &strText, int timeout/* = 0*/)
{
    log (strText);
    this->statusBar()->showMessage (strText, timeout);
}//MainWindow::setStatus

void
MainWindow::onDoBtnClicked ()
{
    if ((Qt::Checked == btnDoText->checkState ()) &&
        (observerText.isNull ()))
    {
        ChannelClassList filters;
        QMap<QString, QDBusVariant> filter;
        filter.insert(
                QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType"),
                QDBusVariant(TELEPATHY_INTERFACE_CHANNEL_TYPE_TEXT));
        filter.insert(
                QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandleType"),
                QDBusVariant((uint) Tp::HandleTypeContact));
        filters.append(filter);
        MyObserver *myobserver = new MyObserver(filters);
        observerText = (MyObserverPtr) myobserver;
        AbstractClientPtr appr = (AbstractClientPtr) myobserver;
        clientRegistrar->registerClient(appr, "MyTextObserver");

        QObject::connect (
                myobserver, SIGNAL (log(const QString &, int)),
                this      , SLOT   (log(const QString &, int)));
        QObject::connect (
                myobserver, SIGNAL (status(const QString &, int)),
                this      , SLOT   (setStatus(const QString &, int)));

        log ("Observing text");
    }
    else if ((Qt::Checked != btnDoText->checkState ()) &&
             (!observerText.isNull ()))
    {
        AbstractClientPtr appr = (AbstractClientPtr) observerText.data ();
        clientRegistrar->unregisterClient (appr);
        observerText->deleteLater ();
        observerText.reset ();
        log ("Stopped observing text");
    }

    if ((Qt::Checked == btnDoStream->checkState ()) &&
        (observerStream.isNull ()))
    {
        ChannelClassList filters;
        QMap<QString, QDBusVariant> filter;
        filter.insert(
                QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType"),
                QDBusVariant(TELEPATHY_INTERFACE_CHANNEL_TYPE_STREAMED_MEDIA));
        filter.insert(
                QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandleType"),
                QDBusVariant((uint) Tp::HandleTypeContact));
        filters.append(filter);
        MyObserver *myobserver = new MyObserver(filters);
        observerStream = (MyObserverPtr) myobserver;
        AbstractClientPtr appr = (AbstractClientPtr) myobserver;
        clientRegistrar->registerClient(appr, "MyStreamObserver");

        QObject::connect (
                myobserver, SIGNAL (log(const QString &, int)),
                this      , SLOT   (log(const QString &, int)));
        QObject::connect (
                myobserver, SIGNAL (status(const QString &, int)),
                this      , SLOT   (setStatus(const QString &, int)));
        log ("Observing streamed media");
    }
    else if ((Qt::Checked != btnDoStream->checkState ()) &&
             (!observerStream.isNull ()))
    {
        AbstractClientPtr appr = (AbstractClientPtr) observerStream.data ();
        clientRegistrar->unregisterClient (appr);
        observerStream->deleteLater ();
        observerStream.reset ();
        log ("Stopped observing streamed media");
    }
}//MainWindow::onDoBtnClicked

void
MainWindow::onActionSelected (int id)
{
    if (!observerText.isNull ())
    {
        observerText->setId (id);
    }

    if (!observerStream.isNull ())
    {
        observerStream->setId (id);
    }
}//MainWindow::onActionSelected
